package de.fhw.swtvertiefung.knauf.kuchenzutatbmp;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

/**
 * Bean-Klasse fr ein Objekt "Kuchen".
 * Hier wird mit BeanManaged Persistence gearbeitet, d.h. alles wird von der Bean verwaltet.
 * Ein Kuchen besteht  aus zwei Feldern:
 *  -ID
 *  -Kuchen-Name
 *
 * Auerdem kommt noch die Zutatenliste hinzu.
 *
 * Ziel dieser Klasse: sie soll sich exakt so verhalten wie die Container-Managed Klassen.
 *
 * @author  Wolfgang Knauf
 * @version 1.0
 */

public class KuchenBMPBean implements EntityBean
{
  /**Tabellename fr Kuchen-Tabelle */
  private static final String TABLENAME = "KuchenBMPBean";

  /**Name der Spalte "KuchenID" */
  private static final String COL_ID = "kuchenid";
  /**Name der Spalte "Kuchen-Name" */
  private static final String COL_NAME = "name";

  /**JNDI-Name der Zutat-Bean */
  private static final String JNDI_ZUTAT = "java:comp/env/ejb/ZutatBMP";

  /**Der Entity-Context */
  private EntityContext entityContext = null;

  /**Der Logger fr diese Klasse. */
  private static final Logger logger = Logger.getLogger ( KuchenBMPBean.class.getName() );

  /**Neu bei Bean Managed Persistence: Alle Felder mssen in der Klasse deklariert sein.
   * Dies ist das Feld "ID".
   */
  private Long lngID = null;

  /**Datenbankfeld fr den Namen des Kuchens.   */
  private String strName = null;

  /**Die Collection der Zutaten, eine Liste von Remote-Interfaces. */
  //private Collection collZutaten = null;

  /**Konstruktor.
   *
   */
  public KuchenBMPBean()
  {
  }

  ////////////////////////////////////////////////////////////////////////////
  //              Methoden aus dem Interface "EntityBean"                   //
  ////////////////////////////////////////////////////////////////////////////

  /**Bean wird aktiviert: Den Primary Key aus dem SessionContext holen
   * und in die Membervariable schreiben.
   */
  public void ejbActivate()
  {
    KuchenBMPBean.logger.info("ejbActivate, in EntityContext steckt dieser Primary Key: " + this.entityContext.getPrimaryKey());

    this.lngID = (Long) this.entityContext.getPrimaryKey();
  }

  /**Laden des Kuchens aus der Datenbank.
   * Der Primary Key wurde schon beim Activate gesetzt.
   */
  public void ejbLoad()
  {
    KuchenBMPBean.logger.info("ejbLoad fr Kuchen " + this.entityContext.getPrimaryKey() );
    //PrimaryKey aus EntityContext holen:
    this.lngID = (Long) this.entityContext.getPrimaryKey();

    try
    {
      //SQL-Select vorbereiten. Die ID ist bekannt, nur den Namen laden.
      String queryString = "select " + COL_NAME + " from " + TABLENAME + " where " + COL_ID + " = ?";

      Connection connLoad = this.getConnection();
      try
      {
        KuchenBMPBean.logger.info("about to execute: " + queryString);
        PreparedStatement preparedStatement = connLoad.prepareStatement(queryString);
        preparedStatement.setLong(1, this.getId().longValue() );
        ResultSet resultSet = preparedStatement.executeQuery();
        while (resultSet.next())
        {
          this.strName = resultSet.getString(1);
        }
        resultSet.close();
        KuchenBMPBean.logger.info("Kuchen " + lngID + " wurde geladen.");
      }
      finally
      {
        connLoad.close();
      }

      KuchenBMPBean.logger.info( "Kuchen loaded, kuchenid = " + this.getId());
    }
    catch (SQLException ex)
    {
      KuchenBMPBean.logger.log(Level.SEVERE, ex.getMessage(), ex);
      throw new EJBException(ex);
    }
  }

  /**Die Bean wird passiviert. Den Primary Key auf NULL setzen.
   *
   */
  public void ejbPassivate()
  {
    KuchenBMPBean.logger.info("ejbPassivate fr Kuchen " + this.lngID);
    this.lngID = null;
  }

  /**Lschen des Kuchens.
   * Entfernt auerdem alle Zutaten.
   */
  public void ejbRemove()
  {
    KuchenBMPBean.logger.info("ejbRemove fr Kuchen " + this.lngID);

    try
    {
      //Zuerst die Zutaten entfernen:
      Collection collZutaten = this.getZutatenListe();
      Iterator iteratorZutaten = collZutaten.iterator();
      while (iteratorZutaten.hasNext() == true)
      {
        ZutatBMPLocal localZutat = (ZutatBMPLocal) iteratorZutaten.next();
        KuchenBMPBean.logger.info("Lsche Zutat " + localZutat.getId() );
        localZutat.remove();
        //Bei der Erfolgs-Logausgabe nicht mehr auf die ID zugreifen, die ist ungltig.
        //Wenn man es doch tut, dann versucht der Container
        //ein Objekt zu laden und fhrt dabei gegen die Wand !
        KuchenBMPBean.logger.info("Zutat erfolgreich gelscht");
      }
      //Jetzt das eigentliche Lschen.
      Connection connRemove = this.getConnection();
      try
      {
        String strQuery = "delete from " + TABLENAME + " where " + COL_ID + " = " + this.getId();
        KuchenBMPBean.logger.info("about to execute: " + strQuery);
        Statement statement = connRemove.createStatement();
        int intCount = statement.executeUpdate(strQuery);
        //Wurde ein Kuchen gelscht ?
        KuchenBMPBean.logger.info("Gelschte Datenstze: " + intCount);
        if (intCount != 1)
          throw new SQLException ("Es wurde nicht genau ein Datensatz gelscht: " + intCount);
      }
      finally
      {
        connRemove.close();
      }
    }
    catch (RemoveException rex)
    {
      throw new EJBException ("Fehler beim Lschen fr ID " + this.getId() + ": " + rex.getMessage(), rex );
    }
    catch (SQLException sex)
    {
      throw new EJBException ("Fehler beim Lschen fr ID " + this.getId() + ": " + sex.getMessage(), sex);
    }
  }

  /**Speichern des Kuchens in der Datenbank.
   * Beim Create erfolgte auf jeden Fall ein Insert, hier also "nur" ein Update.
   */
  public void ejbStore()
  {
    KuchenBMPBean.logger.info("ejbStore fr Kuchen " + this.getId() );
    try
    {
      String updateString = "update " + TABLENAME + " set " + COL_NAME + " = ? where " + COL_ID + " = ?";

      Connection connLoad = this.getConnection();
      try
      {
        KuchenBMPBean.logger.info ("about to execute: " + updateString);
        PreparedStatement preparedStatement = connLoad.prepareStatement(updateString);
        preparedStatement.setString(1, this.getName() );
        preparedStatement.setLong(2, this.getId().longValue() );
        if (1 != preparedStatement.executeUpdate())
        {
          throw new SQLException("Update failed. No row for kuchenid = " + this.getId() );
        }
        KuchenBMPBean.logger.info("update ok.");
      }
      finally
      {
        connLoad.close();
      }

      KuchenBMPBean.logger.info ("Kuchen stored, kuchenid = " + this.getId() );
    }
    catch (SQLException ex)
    {
      KuchenBMPBean.logger.log(Level.SEVERE, ex.getMessage(), ex);
      throw new EJBException(ex);
    }
  }

  /**Entity-Context wird zurckgesetzt.
   *
   */
  public void unsetEntityContext()
  {
    KuchenBMPBean.logger.info("unsetEntityContext");
    this.entityContext = null;
  }

  /**Entity-Context wird gesetzt: Speichern in Klassenvariablen.
   * @param entityContext Der Context
   */
  public void setEntityContext(EntityContext entityContext)
  {
    KuchenBMPBean.logger.info("setEntityContext");
    this.entityContext = entityContext;
  }


  //////////////////////////////////////////////////////////////////////////////////
  //                   Zugriffsmethoden fr die Felder                            //
  //////////////////////////////////////////////////////////////////////////////////

  /**Abrufen der Kuchen-ID. Darf hier nicht abstrakt sein.
   * Es gibt keine Set-Methode dazu (weil es das PrimaryKey-Feld ist).
   * @return Die Kuchen-ID.
   */
  public Long getId()
  {
    return this.lngID;
  }


  /**Abrufen des Kuchen-Namens. Darf hier nicht abstrakt sein.
   * @return der Kuchen-Name.
   */
  public String getName()
  {
    return this.strName;
  }

  /**Setzen des Kuchen-Namens. Darf hier nicht abstrakt sein.
   * @param str_Name der Name
   */
  public void setName (String str_Name)
  {
    this.strName = str_Name;
  }

  /**Abrufen aller Zutaten dieses Kuchens.
   * D.h. eine Collection der Local-Interfaces.
   * @return Collection von Zutat-LocalInterfaces.
   */
  public Collection getZutaten()
  {
    try
    {
      //Holen des Kuchen-Home-Interfaces.
      InitialContext initialContext = new InitialContext();
      Object homeref = initialContext.lookup ( JNDI_ZUTAT );

      //Wegen LocalInterface kein "PortableObject.narrow()" ntig.
      ZutatBMPLocalHome zutatHome = (ZutatBMPLocalHome) homeref;

      return zutatHome.findByKuchenId ( this.getId() );
    }
    catch (NamingException nex)
    {
      throw new EJBException ("Fehler beim Laden der Zutaten zu Kuchen: " + nex.getMessage(), nex);
    }
    catch (FinderException fex)
    {
      throw new EJBException ("Fehler beim Laden der Zutaten zu Kuchen: " + fex.getMessage(), fex);
    }
  }

  /**Hinzufgen einer Zutat zu den Zutaten dieses Kuchens (die einzige Business-Methode).
   * Hier werden die relevanten Felder der Zutat bergeben, die Bean setzt selbst die Kuchen-Id.
   * Alle Fehler werden in EJBExceptions umgewandelt.
   * @param str_ZutatName Name der Zutat.
   * @param str_Menge Erforderliche Menge
   * @return Die erzeugte Zutat.
   */
  public ZutatBMPLocal addZutat (String str_ZutatName, String str_Menge)
  {
    //Erzeugen der Zutat.
    ZutatBMPLocalHome zutatHome = null;
    try
    {
      //Das Home-Interface knnte man natrlich cachen.
      InitialContext initialContext = new InitialContext();
      Object homeref = initialContext.lookup ( JNDI_ZUTAT);
      //Wegen LocalInterface kein "PortableObject.narrow()" ntig.
      zutatHome = (ZutatBMPLocalHome) homeref;
    }
    catch (javax.naming.NamingException nex)
    {
      throw new EJBException ("Fehler beim Lookup der ZutatBean-Reference: " + nex.getMessage(), nex);
    }

    try
    {
      //Durch das Create ist die Zutat fertig.
      ZutatBMPLocal zutat = zutatHome.createByNameMengeKuchenId(str_ZutatName, str_Menge, this.getId() );

      return zutat;
    }
    catch (CreateException cex)
    {
      throw new EJBException ("Fehler beim Erzeugen der ZutatBean: " + cex.getMessage(), cex);
    }
  }

  /**Abrufen der Zutatenliste.
   * Da hier mit BeanManagedPersistence gearbeitet wird, wird die Liste manuell als Liste von LocalInterfaces
   * erzeugt.
   * Fraglich ist, ob es auch mglich wre, nur mit "getZutaten" zu arbeiten, da dort ebenfalls die LocalInterfaces
   * manuell erzeugt werden.
   * @return Eine Liste aller Zutaten.
   */
  public ArrayList getZutatenListe()
  {
    ArrayList list = new ArrayList();
    Iterator c = this.getZutaten().iterator();
    while (c.hasNext())
    {
      list.add((ZutatBMPLocal)c.next());
    }
    return list;
  }

  //////////////////////////////////////////////////////////////////////////////////
  //                          Weitere Methoden                                    //
  //////////////////////////////////////////////////////////////////////////////////

  /**Erzeugen eines Kuchen-Objekts, wobei der Name angegeben wird. Der Key wird automtatisch erzeugt
   * und der Datensatz gespeichert.
   * @param str_Name Der Name des Kuchens.
   * @return Die neue ID des Satzes.
   * @exception CreateException Fehler beim Erzeugen.
   */
  public Long ejbCreateByName (String str_Name) throws CreateException
  {
    KuchenBMPBean.logger.info("ejbCreateByName: " + str_Name);
    //Die ID erzeugen:
    this.lngID = this.ejbHomeGetNextId();
    this.setName(str_Name);

    //INSERT ausfhren:
    try
    {
      String insertString = "insert into " + TABLENAME + " (" + COL_ID + ", " + COL_NAME + ") values (?, ?)";

      Connection connInsert = this.getConnection();
      try
      {
        KuchenBMPBean.logger.info ("about to execute: " + insertString);
        PreparedStatement preparedStatement = connInsert.prepareStatement(insertString);
        preparedStatement.setLong(1, this.getId().longValue() );
        preparedStatement.setString(2, this.getName() );
        preparedStatement.executeUpdate();

        KuchenBMPBean.logger.info("Insert ok.");
      }
      finally
      {
        connInsert.close();
      }

      KuchenBMPBean.logger.info ("Kuchen inserted, kuchenid = " + this.getId() );
    }
    catch (SQLException ex)
    {
      KuchenBMPBean.logger.log(Level.SEVERE, ex.getMessage(), ex);
      throw new EJBException(ex);
    }

    return this.getId();
  }

  /**Nach dem Erzeugen eines Kuchen-Objekts, wobei der Name angegeben wird.
   * @param str_Name Der Name des Kuchens.
   * @exception CreateException Fehler beim Erzeugen.
   */
  public void ejbPostCreateByName (String str_Name) throws CreateException
  {
    KuchenBMPBean.logger.info("ejbPostCreateByName: " + str_Name);
  }

  /**Suchen eines Kuchens anhand des Primary Keys.
   * Die Methode prft nur, ob der Kuchen in der Datenbank vorhanden ist !
   * @param lng_PrimaryKey Die ID des zu ladenden Kuchens.
   * @return Den bergebenen PrimaryKey, oder NULL, falls nicht gefunden.
   * @exception FinderException Falls kein Datensatz gefunden wurde oder ein Fehler auftrat.
   */
  public Long ejbFindByPrimaryKey (Long lng_PrimaryKey) throws FinderException
  {
    boolean bolFound = false;
    try
    {
      Connection connFind = this.getConnection();
      try
      {
        String strQuery = "select " + COL_ID + " from " + TABLENAME + " where " + COL_ID + " = " + lng_PrimaryKey;
        KuchenBMPBean.logger.info("about to execute: " + strQuery);
        Statement statement = connFind.createStatement();
        ResultSet resultSet = statement.executeQuery(strQuery);
        //Gibt es einen Datensatz zur ID ?
        bolFound = resultSet.next();
        resultSet.close();
        KuchenBMPBean.logger.info("Datensatz gefunden ?: " + bolFound);
      }
      finally
      {
        connFind.close();
      }
    }
    catch (SQLException e)
    {
      throw new EJBException ("Fehler beim FindByPrimaryKey fr ID " + lng_PrimaryKey);
    }

    //Rckgabe: Key oder NULL.
    if (bolFound == true)
      return lng_PrimaryKey;
    else
      return null;
  }

  /**Von auen verwendbare Methode aus dem Home-Interface: Ermitteln der nchsten verfgbaren
   * Kuchen-ID.
   * @return Die neue Kuchen-ID, 1 grer als die maximal vorhandene ID. Wenn es keine gibt,
   * wird "1" zurckgeliefert.
   */
  public Long ejbHomeGetNextId()
  {
    Long lngMaxID = null;

    String queryString = "select max (" + COL_ID + ") from " + TABLENAME;

    try
    {
      Connection connLoad = this.getConnection();
      try
      {
        KuchenBMPBean.logger.info("about to execute: " + queryString);
        Statement statement = connLoad.createStatement();
        ResultSet resultSet = statement.executeQuery(queryString);
        while (resultSet.next())
        {
          lngMaxID = new Long( resultSet.getLong(1) );
        }
        resultSet.close();
        KuchenBMPBean.logger.info("Maximale ID: " + lngMaxID);
      }
      finally
      {
        connLoad.close();
      }
    }
    catch (SQLException ex)
    {
      KuchenBMPBean.logger.log(Level.SEVERE, ex.getMessage(), ex);
      throw new EJBException(ex);
    }

    //Der aktuell maximale Key wurde aus Datenbank geholt.  Nun um 1 erhhen.
    if (lngMaxID == null)
      lngMaxID = new Long(0);
    //Um 1 erhhen:
    return new Long (lngMaxID.longValue() + 1);
  }

  /**Einlesen aller Kuchen.
   * @return Eine Collection der >>>>IDs<<<< aller Kuchen-Objekte (nicht der LocalHome-Interfaces).
   * @exception FinderException Falls kein Datensatz gefunden wurde oder ein Fehler auftrat.
   */
  public Collection ejbFindAll () throws FinderException
  {
    ArrayList collReturn = new ArrayList ();
    try
    {
      Connection connFind = this.getConnection();
      try
      {
        //Alle IDs laden.
        String strQuery = "select " + COL_ID + " from " + TABLENAME + " order by " + COL_NAME;
        KuchenBMPBean.logger.info("about to execute: " + strQuery);
        Statement statement = connFind.createStatement();
        ResultSet resultSet = statement.executeQuery(strQuery);
        //Gibt es einen Datensatz zur ID ?
        while (resultSet.next() == true)
        {
          long lngID = resultSet.getLong(1);
          KuchenBMPBean.logger.info("verarbeite Kuchen " + lngID);

          //Nur die ID ist die Rckgabe ! Der Container ldt selbst die Einzelobjekte dazu.
          collReturn.add (new Long (lngID) );
        }
        resultSet.close();
        KuchenBMPBean.logger.info("Datenstze gefunden ?: " + collReturn.size() );
      }
      finally
      {
        connFind.close();
      }
    }
    catch (SQLException e)
    {
      throw new EJBException ("SQLException beim Laden aller Kuchen: " + e.getMessage(), e);
    }

    return collReturn;
  }

  //////////////////////////////////////////////////////////////////////////
  //             Hilfsmethoden                                            //
  //////////////////////////////////////////////////////////////////////////

  /**Erzeugen einer Datenbankconnection.
   * Holt die DataSource aus dem Context der Bean und erstellt eine Connection.
   * Diese Methode steht identisch in der Klasse "ZutatBeanBMP" !
   * @return Die erzeugte Connection.
   * @exception EJBException Gekapselte Fehler beim Zugriff auf Datenbank oder JNDI.
   */
  private Connection getConnection() throws EJBException
  {
    try
    {
      InitialContext initialContext = new InitialContext();

      DataSource dataSource = (DataSource) initialContext.lookup("java:comp/env/jdbc/KuchenZutatBMPDB");

      return dataSource.getConnection();
    }
    catch (SQLException sex)
    {
      throw new EJBException ("Fehler beim Erzeugen einer Connection: " + sex.getMessage(), sex);
    }
    catch (NamingException nex)
    {
      throw new EJBException ("Fehler beim JNDI-Zugriff beim Erzeugen einer Connection: " + nex.getMessage(), nex);
    }
  }
}