package de.fhw.swtvertiefung.knauf.kuchenzutatbmp;

import java.rmi.RemoteException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collection;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityContext;
import javax.ejb.EntityBean;
import javax.ejb.FinderException;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

/**
 * Diese Bean bildet eine Zutat fr den Kuchen ab. <br>
 * Werte sind: <br>
 * a) ID-Spalte (Primary Key) <br>
 * b) Name der Zutat <br>
 * c) Menge <br>
 * d) Foreign Key auf einen Kuchen. <br>
 * Der Kuchen wird Bean-Managed verwaltet, d.h. die gesamte Logik geht ber diese Klasse. <br>
 * 
 * @author  Wolfgang Knauf
 * @version 1.0
 */

public class ZutatBMPBean implements EntityBean
{
  /**Tabellename fr Zutaten-Tabelle */
  private static final String TABLENAME = "ZutatBMPBean";

  /**Name der Spalte "ZutatID" */
  private static final String COL_ID = "zutatid";
  /**Name der Spalte "Zutat-Name" */
  private static final String COL_NAME = "zutatName";
  /**Name der Spalte "Menge" */
  private static final String COL_MENGE = "menge";
  /**Name der Spalte "Kuchen-ID" */
  private static final String COL_KUCHENID = "kuchenid";

  /**JNDI-Name der Kuchen-Bean */
  private static final String JNDI_KUCHEN = "java:comp/env/ejb/KuchenBMP";

  /**Der EntityContext */
  private EntityContext entityContext = null;

  /**Der Logger fr Ausgabe dieser Klasse. */
  private static final Logger logger = Logger.getLogger ( ZutatBMPBean.class.getName() );

  /**Dies ist das Feld "ID".
   */
  private Long lngId = null;

  /**Datenbankfeld fr den Namen der Zutat.   */
  private String strZutatName = null;

  /**Datenbankfeld fr die Menge.   */
  private String strMenge = null;

  /**Foreign Key auf den zugehrigen Kuchen */
  private Long lngKuchenId = null;

  /**Konstruktor. Macht nichts.
   *
   */
  public ZutatBMPBean()
  {
  }

  ////////////////////////////////////////////////////////////////////////////
  //              Methoden aus dem Interface "EntityBean"                   //
  ////////////////////////////////////////////////////////////////////////////

  /**Bean wird aktiviert: Den Primary Key aus dem SessionContext holen
   * und in die Membervariable schreiben.
   */
  public void ejbActivate()
  {
    ZutatBMPBean.logger.info("ejbActivate, in EntityContext steckt dieser Primary Key: " + this.entityContext.getPrimaryKey());

    this.lngId = (Long) this.entityContext.getPrimaryKey();
  }

  /**Laden der Zutat aus der Datenbank.
   * Der Primary Key wurde schon beim Activate gesetzt.
   */
  public void ejbLoad()
  {
    ZutatBMPBean.logger.info("ejbLoad fr Zutat " + this.entityContext.getPrimaryKey() );
    //PrimaryKey aus EntityContext holen:
    this.lngId = (Long) this.entityContext.getPrimaryKey();

    try
    {
      //SQL-Select vorbereiten. Die ID ist bekannt, nur den Namen laden.
      String queryString = "select " + COL_NAME + ", " + COL_MENGE + ", " + COL_KUCHENID + " from " + TABLENAME + " where " + COL_ID + " = " + this.lngId;

      Connection connLoad = this.getConnection();
      try
      {
        ZutatBMPBean.logger.info("about to execute: " + queryString);
        Statement statement = connLoad.createStatement();
        ResultSet resultSet = statement.executeQuery(queryString);
        while (resultSet.next())
        {
          this.strZutatName = resultSet.getString(COL_NAME);
          this.strMenge = resultSet.getString (COL_MENGE);
          this.lngKuchenId = new Long( resultSet.getLong(COL_KUCHENID) );
        }
        resultSet.close();
        ZutatBMPBean.logger.info("Zutat " + lngId + " wurde geladen.");
      }
      finally
      {
        connLoad.close();
      }

      ZutatBMPBean.logger.info( "Zutat loaded, zutatid = " + this.getId());
    }
    catch (SQLException ex)
    {
      ZutatBMPBean.logger.log(Level.SEVERE, ex.getMessage(), ex);
      throw new EJBException(ex);
    }
  }

  /**Die Bean wird passiviert. Den Primary Key auf NULL setzen.
   *
   */
  public void ejbPassivate()
  {
    ZutatBMPBean.logger.info("ejbPassivate fr Zutat " + this.lngId);
    this.lngId = null;
  }

  /**Lschen der Zutat.
   * Die Abhngigkeit zum Kuchen wird natrlich NICHT gelscht.
   */
  public void ejbRemove()
  {
    ZutatBMPBean.logger.info("ejbRemove fr Zutat " + this.lngId);

    try
    {
      Connection connRemove = this.getConnection();
      try
      {
        String strQuery = "delete from " + TABLENAME + " where " + COL_ID + " = " + this.getId();
        ZutatBMPBean.logger.info("about to execute: " + strQuery);
        Statement statement = connRemove.createStatement();
        int intCount = statement.executeUpdate(strQuery);
        //Wurde eine Zutat gelscht ?
        ZutatBMPBean.logger.info("Gelschte Datenstze: " + intCount);
        if (intCount != 1)
          throw new SQLException ("Es wurde nicht genau ein Datensatz gelscht: " + intCount);
      }
      finally
      {
        connRemove.close();
      }
    }
    catch (SQLException e)
    {
      throw new EJBException ("Fehler beim Lschen fr ID " + this.getId());
    }
  }

  /**Speichern der Zutat in der Datenbank.
   * Beim Create erfolgte auf jeden Fall ein Insert, hier also "nur" ein Update.
   */
  public void ejbStore()
  {
    ZutatBMPBean.logger.info("ejbStore fr Zutat " + this.getId() );
    try
    {
      String updateString = "update " + TABLENAME + " set " + COL_NAME + " = ?, " + COL_MENGE + " = ?, " + COL_KUCHENID + " = " + this.lngKuchenId + " where " + COL_ID + " = " + this.getId();

      Connection connLoad = this.getConnection();
      try
      {
        ZutatBMPBean.logger.info ("about to execute: " + updateString);
        PreparedStatement preparedStatement = connLoad.prepareStatement(updateString);
        preparedStatement.setString (1, this.getZutatName() );
        preparedStatement.setString (2, this.getMenge() );
        if (1 != preparedStatement.executeUpdate())
        {
          throw new SQLException("Update failed. No row for zutatid = " + this.getId() );
        }
        ZutatBMPBean.logger.info("update ok.");
      }
      finally
      {
        connLoad.close();
      }

      ZutatBMPBean.logger.info ("Zutat stored, zutatid = " + this.getId() );
    }
    catch (SQLException ex)
    {
      ZutatBMPBean.logger.log(Level.SEVERE, ex.getMessage(), ex);
      throw new EJBException(ex);
    }
  }

  /**Entity-Context wird zurckgesetzt.
   *
   */
  public void unsetEntityContext()
  {
    ZutatBMPBean.logger.info("unsetEntityContext");
    this.entityContext = null;
  }

  /**Entity-Context wird gesetzt: Speichern in Klassenvariablen.
   * @param entityContext Der Context
   */
  public void setEntityContext(EntityContext entityContext)
  {
    ZutatBMPBean.logger.info("setEntityContext");
    this.entityContext = entityContext;
  }

  //////////////////////////////////////////////////////////////////////////////////
  //                   Zugriffsmethoden fr die Felder                            //
  //////////////////////////////////////////////////////////////////////////////////

  /**Abrufen der Zutat-ID. Darf hier nicht abstrakt sein.
   * Es gibt keine Set-Methode dazu (weil es das PrimaryKey-Feld ist).
   * @return Die Zutat-ID.
   */
  public Long getId()
  {
    return this.lngId;
  }

  /**Abrufen des Zutat-Namens.
   * @return der Zutat-Name.
   */
  public String getZutatName()
  {
    return this.strZutatName;
  }

  /**Setzen des Zutat-Namens. Darf hier nicht abstrakt sein.
   * @param str_ZutatName Der Name der Zutat
   */
  public void setZutatName (String str_ZutatName)
  {
    this.strZutatName = str_ZutatName;
  }

  /**Abrufen der Menge
   * @return Die Menge.
   */
  public String getMenge()
  {
    return this.strMenge;
  }

  /**Setzen der Menge. Darf hier nicht abstrakt sein.
   * @param str_Menge Die bentigte Menge
   */
  public void setMenge (String str_Menge)
  {
    this.strMenge = str_Menge;
  }

  /**Abrufen der Kuchen-ID
   * @return Die Kuchen-ID.
   */
  public Long getKuchenId()
  {
    return this.lngKuchenId;
  }

  /**Setzen der Kuchen-ID. Darf hier nicht abstrakt sein.
   * @param lng_KuchenId Die Kuchen-ID
   */
  public void setKuchenId (Long lng_KuchenId)
  {
    this.lngKuchenId = lng_KuchenId;
  }

  /**Abrufen des Kuchens, zu dem die Zutat gehrt.
   * Fhrt ein "findByPrimaryKey" auf das HomeInterface des Kuchens zurck.
   * @return Der Kuchen der Zutat.
   */
  public KuchenBMPLocal getKuchen()
  {
    KuchenBMPLocalHome kuchenHome = null;
    try
    {
      //Das Home-Interface knnte man natrlich cachen.
      InitialContext initialContext = new InitialContext();
      Object homeref = initialContext.lookup (JNDI_KUCHEN);
      //Wegen LocalInterface kein "PortableObject.narrow()" ntig.
      kuchenHome = (KuchenBMPLocalHome) homeref;
    }
    catch (javax.naming.NamingException nex)
    {
      throw new EJBException ("Fehler beim Lookup der KuchenBean-Reference: " + nex.getMessage(), nex);
    }

    try
    {
      //Suchen des Kuchens.
      KuchenBMPLocal kuchen = kuchenHome.findByPrimaryKey(this.getKuchenId() );

      return kuchen;
    }
    catch (FinderException fex)
    {
      throw new EJBException ("Fehler beim Suchen des Kuchens fr ID " + this.getKuchenId() + ": " + fex.getMessage(), fex);
    }
  }

  //////////////////////////////////////////////////////////////////////////////////
  //                          Weitere Methoden                                    //
  //////////////////////////////////////////////////////////////////////////////////

  /**Erzeugen eines Zutat-Objekts, wobei Name, Menge und Kuchen-ID angegeben wird. Der Key wird in der Methode
   * erzeugt.
   * @param str_Name Der Name der Zutat
   * @param str_Menge Die Menge der Zutat
   * @param lng_KuchenId Die ID des zugehrigen Kuchens.
   * @return Die ID der Zutat.
   * @exception CreateException Fehler beim Erzeugen.
   */
  public Long ejbCreateByNameMengeKuchenId (String str_Name, String str_Menge, Long lng_KuchenId) throws CreateException
  {
    ZutatBMPBean.logger.info("ejbCreateByNameMengeKuchenId: Name = " + str_Name + ", Menge = " + str_Menge + ", KuchenId = " + lng_KuchenId);
    this.setZutatName(str_Name);
    this.setMenge(str_Menge);
    this.setKuchenId(lng_KuchenId);
    //Die ID erzeugen:
    this.lngId = this.ejbHomeGetNextId();

    //INSERT ausfhren:
    try
    {
      String insertString = "insert into " + TABLENAME + " (" + COL_ID + ", " + COL_NAME + ", " + COL_MENGE + ", " + COL_KUCHENID + ")" +
      " values (" + this.getId() + ", ?, ?, " + this.getKuchenId() + ")";

      Connection connInsert = this.getConnection();
      try
      {
        ZutatBMPBean.logger.info ("about to execute: " + insertString);
        PreparedStatement preparedStatement = connInsert.prepareStatement(insertString);
        preparedStatement.setString(1, this.getZutatName() );
        preparedStatement.setString(2, this.getMenge() );
        preparedStatement.executeUpdate();

        ZutatBMPBean.logger.info("Insert ok.");
      }
      finally
      {
        connInsert.close();
      }

      ZutatBMPBean.logger.info ("ZutatBeanBMP inserted, zutatid = " + this.getId() );
    }
    catch (SQLException ex)
    {
      ZutatBMPBean.logger.log(Level.SEVERE, ex.getMessage(), ex);
      throw new EJBException(ex);
    }

    return this.getId();
  }

  /**Nach dem Erzeugen eines Zutat-Objekts, wobei Name und Menge angegeben werden: nichts besonderes zu tun.
   * @param str_Name Der Name der Zutat.
   * @param str_Menge Menge der Zutat
   * @param lng_KuchenId Die ID des zugehrigen Kuchens.
   * @exception CreateException Fehler beim Erzeugen.
   */
  public void ejbPostCreateByNameMengeKuchenId (String str_Name, String str_Menge, Long lng_KuchenId) throws CreateException
  {
    ZutatBMPBean.logger.info("ejbPostCreateByNameMengeKuchenId: Name = " + str_Name + ", Menge = " + str_Menge + ", KuchenId = " + lng_KuchenId);
  }

  /**Suchen einer Zutat anhand des Primary Keys.
   * Die Methode prft nur, ob die Zutat in der Datenbank vorhanden ist !
   * @param lng_PrimaryKey Die ID der zu ladenden Zutat.
   * @return Den bergebenen PrimaryKey, oder NULL, falls nicht gefunden.
   * @exception FinderException Falls kein Datensatz gefunden wurde oder ein Fehler auftrat.
   */
  public Long ejbFindByPrimaryKey (Long lng_PrimaryKey) throws FinderException
  {
    boolean bolFound = false;
    try
    {
      Connection connFind = this.getConnection();
      try
      {
        String strQuery = "select " + COL_ID + " from " + TABLENAME + " where " + COL_ID + " = " + lng_PrimaryKey;
        ZutatBMPBean.logger.info("about to execute: " + strQuery);
        Statement statement = connFind.createStatement();
        ResultSet resultSet = statement.executeQuery(strQuery);
        //Gibt es einen Datensatz zur ID ?
        bolFound = resultSet.next();
        resultSet.close();
        ZutatBMPBean.logger.info("Datensatz gefunden ?: " + bolFound);
      }
      finally
      {
        connFind.close();
      }
    }
    catch (SQLException e)
    {
      throw new EJBException ("Fehler beim FindByPrimaryKey fr ID " + lng_PrimaryKey);
    }

    //Rckgabe: Key oder NULL.
    if (bolFound == true)
      return lng_PrimaryKey;
    else
      return null;
  }

  /**Ermitteln der nchsten verfgbaren Zutat-ID.
   * @return Die neue Zutat-ID, 1 grer als die maximal vorhandene ID. Wenn es keine gibt,
   * wird "1" zurckgeliefert.
   */
  public Long ejbHomeGetNextId()
  {
    Long lngMaxID = null;

    String queryString = "select max (" + COL_ID + ") from " + TABLENAME;

    try
    {
      Connection connLoad = this.getConnection();
      try
      {
        ZutatBMPBean.logger.info("about to execute: " + queryString);
        Statement statement = connLoad.createStatement();
        ResultSet resultSet = statement.executeQuery(queryString);
        while (resultSet.next())
        {
          lngMaxID = new Long( resultSet.getLong(1) );
        }
        resultSet.close();
        ZutatBMPBean.logger.info("Maximale ID: " + lngMaxID);
      }
      finally
      {
        connLoad.close();
      }
    }
    catch (SQLException ex)
    {
      ZutatBMPBean.logger.log(Level.SEVERE, ex.getMessage(), ex);
      throw new EJBException(ex);
    }

    //Der aktuell maximale Key wurde aus Datenbank geholt.  Nun um 1 erhhen.
    if (lngMaxID == null)
      lngMaxID = new Long(0);
    //Um 1 erhhen:
    return new Long (lngMaxID.longValue() + 1);
  }

  /**Laden aller Zutaten-Objekte eines Kuchens
   * @param lng_KuchenId Fr diesen Kuchen werden die Zutaten gesucht.
   * @return Eine Collection aller Zutaten eines Kuchens.
   * @exception FinderException Fehler beim Finden.
   * @exception RemoteException Bei Netzwerkfehlern.
   */
  public Collection ejbFindByKuchenId(Long lng_KuchenId) throws FinderException
  {
    ArrayList collReturn = new ArrayList ();
    try
    {
      Connection connFind = this.getConnection();
      try
      {
        //Alle IDs laden.
        String strQuery = "select " + COL_ID + " from " + TABLENAME + " where " + COL_KUCHENID + " = " + lng_KuchenId + " order by " + COL_NAME;
        ZutatBMPBean.logger.info("about to execute: " + strQuery);
        Statement statement = connFind.createStatement();
        ResultSet resultSet = statement.executeQuery(strQuery);
        //Gibt es einen Datensatz zur ID ?
        while (resultSet.next() == true)
        {
          long lngID = resultSet.getLong(1);
          ZutatBMPBean.logger.info("verarbeite Zutat " + lngID);

          //Nur die ID ist die Rckgabe ! Der Container ldt selbst die Einzelobjekte dazu.
          collReturn.add (new Long (lngID) );
        }
        resultSet.close();
        ZutatBMPBean.logger.info("Datenstze gefunden ?: " + collReturn.size() );
      }
      finally
      {
        connFind.close();
      }
    }
    catch (SQLException e)
    {
      throw new EJBException ("SQLException beim Laden der Zutaten eines Kuchens: " + e.getMessage(), e);
    }

    return collReturn;
  }

  //////////////////////////////////////////////////////////////////////////
  //             Hilfsmethoden                                            //
  //////////////////////////////////////////////////////////////////////////

  /**Erzeugen einer Datenbankconnection.
   * Diese Methode steht identisch in der Klasse "KuchenBeanBMP" !
   * Holt die DataSource aus dem Context der Bean und erstellt eine Connection.
   * @return Die erzeugte Connection.
   * @exception EJBException Gekapselte Fehler beim Zugriff auf Datenbank oder JNDI.
   */
  private Connection getConnection() throws EJBException
  {
    try
    {
      InitialContext initialContext = new InitialContext();

      DataSource dataSource = (DataSource) initialContext.lookup("java:comp/env/jdbc/KuchenZutatBMPDB");

      return dataSource.getConnection();
    }
    catch (SQLException sex)
    {
      throw new EJBException ("Fehler beim Erzeugen einer Connection: " + sex.getMessage(), sex);
    }
    catch (NamingException nex)
    {
      throw new EJBException ("Fehler beim JNDI-Zugriff beim Erzeugen einer Connection: " + nex.getMessage(), nex);
    }
  }
}